import { _decorator, Component, Node, SpriteComponent, SpriteFrame, AudioSourceComponent, loader, AudioClip, AudioSource, resources } from 'cc';
import GD from '../config/GD';
import GDef from '../config/GDef';
import BaseView from './BaseView';
const { ccclass, property } = _decorator;

@ccclass('AudioMgr')
export class AudioMgr extends BaseView {
    /* class member could be defined like this */
    // dummy = '';

    /* use `property` decorator if your want the member to be serializable */
    // @property
    // serializableDummy = 0;


    private audioMgr: AudioSource = null!;
    private audio: boolean = true;//声音状态
    private _filePath: string = 'audio';
    private soundResouces: AudioClip[] = [];//音频剪辑

    start() {
        this.audioMgr = this.node.getComponent(AudioSource)!;
        this.on(GDef.wdEvent.playEffect, this.playEffect);//播放音效
        this.on(GDef.wdEvent.playBgm, this.playMusic);
        this.on(GDef.wdEvent.voiceSet, this.btn_audioSeting_cb);
        let Audio = GD.playerMgr.Voice;
        if (!Audio) {
            this.audio = false;
        } else {
            this.audio = true;
        }
        this.loadAudio();
    }

    /**
   * 加载音频文件
   */
    loadAudio() {
        resources.loadDir(this._filePath, AudioClip, (error, resources:any) => {
            if (error) {
                console.log('load sound error: ', error);
                return;
            }
            for (let index = 0; index < resources.length; index++) {
                let soundResouce = resources[index];
                this.soundResouces[soundResouce._name] = soundResouce;
            }
            this.scheduleOnce(() => {
                this.playMusic(true);
            }, 1);
            console.log('load sound resources success');
        })

    }

    /**
      * 声音设置
      */
    private btn_audioSeting_cb(bool:boolean) {
        this.audio = bool;
        // GD.playerMgr.Voice = this.audio;
        this.pause();//播放或暂停声音
        // this.playEffect("btn");
    }

    /**
   * @description 暂停声音
   */
    public pause() {
        if (!this.audio) {
            this.audioMgr.pause();
        } else {
            this.audioMgr.play();
        }
    }


    /**
       * @description 播放音效
       * @param {string} fileName
       */
    public playEffect(fileName: string) {
        let audioClip = this.isCanPlayAudio(fileName);
        if (audioClip) {
            this.audioMgr.playOneShot(audioClip);
        }

    }
    /**
       * @description 播放音乐（默认循环）
       * @param {string} fileName
       */
    public playMusic(bool: boolean) {
        let fileName = "bgm"
        let audioClicp = this.isCanPlayAudio(fileName);
        if (audioClicp) {
            this.audioMgr.clip = audioClicp;
            if (bool) {
                this.audioMgr.play();
            } else {
                this.audioMgr.stop();
            }
        }
    }
    /**
  * @description 播放声音
  * @param fileName
  */
    private isCanPlayAudio(fileName: any):AudioClip {
        if (!this.audio) {
            console.log("处于静音");
            return null!;
        }
        let file = this.soundResouces[fileName];
        if (!file) {
            console.log('sound: fieName is error, please check it');
            return null!;
        }
        return file;
    }
    // update (deltaTime: number) {
    //     // Your update function goes here.
    // }
}
