import { _decorator, Component, Node, ButtonComponent, Prefab, Color, instantiate, Vec3, math } from "cc";
import { Notifications } from "../easyFramework/notifications";
import GDef from "../config/GDef";
import global, { CHANNEL } from "../config/global";
import { Toutiao } from "../easyFramework/channel/toutiao/toutiao";
import { Wechat } from "../easyFramework/channel/wechat/wechat";
import GD from "../config/GD";
const { ccclass, property } = _decorator;

@ccclass("BaseView")
export default class BaseView extends Component {
    private _clickEnable: boolean = true; // 控制能否点击的变量
    private _onEvent: boolean = false;//判断是否有事件监听
    /**
     * 添加事件
     * @param {string} eKey     识别key
     * @param {object} eCall    回调函数
     * @param {object} self     识别对象
     */
    protected on(eKey:string, eCall:any, self?:Object) {
        Notifications.on(eKey, eCall, self ? self : this);
        this._onEvent = true;
    }
    /**
     * 通知事件
     * @param {string} eKey     识别key
     * @param {object} data     传值
     * @param {object} data2     传值
     */
    protected emit(eKey:string, ...parameters: any[]) {
        let a = Notifications.emit(eKey, ...parameters);
        if (a) {
            return a;
        }
    }

    /**
    * 注销某个事件
    * @param {string} eKey 识别key
    * @param {object} self 识别对象 
    */
    protected off(eKey:string, self?:Object) {
        Notifications.off(eKey, self ? self : this);
    }


    /**
     * 绑定按钮点击事件,永久监听
     * @param node 挂载了button组件的节点
     * @param callback 回调
     * @param opname 操作内容
     */
    protected bindButton(node: Node, callback: (...parameters: any) => void, ...parameters: any[]): void {
        let btnComp = node.getComponent(ButtonComponent);
        if (!btnComp) {
            btnComp = node.addComponent(ButtonComponent);
            btnComp.transition = ButtonComponent.Transition.SCALE
        }
        btnComp.zoomScale = 0.9;
        let btnCb = (event:any) => {
            this.emit(GDef.wdEvent.playEffect, "btn");
            if (!this._clickEnable) return;
            this._clickEnable = false;
            this.scheduleOnce(() => {
                this._clickEnable = true;
            }, 0.3);
            callback.call(this, ...parameters);
            TyqSdk.outMain();
        };
        node.on("click", btnCb, this);
    }

    /**
    * 绑定按钮点击事件(只监听一次)
    * @param node 挂载了button组件的节点
    * @param callback 回调
    * @param opname 操作内容
    */
    protected bindButtonOce(node: Node, callback: (event?: any) => void, opname?: string): void {
        let btnComp = node.getComponent(ButtonComponent);
        if (!btnComp) {
            btnComp = node.addComponent(ButtonComponent);
            btnComp.transition = ButtonComponent.Transition.SCALE
        }
        btnComp.zoomScale = 0.9;
        let btnCb = (event:any) => {
            if (!this._clickEnable) return;
            this._clickEnable = false;
            this.scheduleOnce(() => {
                this._clickEnable = true;
            }, 0.3);
            callback.call(this, event);
        };
        node.once("click", btnCb, this);
    }


    onDestroy() {
        if (this._onEvent) {
            Notifications.offTarget(this);
        }
    }

    /**
     * 加载预制体，UI都用这个方法
     * @param cPrefab 要生成的预制体
     * @param cNode 父节点
     * @param next 返回预制体的主脚本
     */
    showUI(cPrefab: Prefab, cNode: Node, next?:any) {
        let node: Node = instantiate(cPrefab);
        node.parent = cNode;
        next && next(node.getComponent(node.name) ? node.getComponent(node.name) : node);
    }

    /**震动 */
    protected shock() {
        const bool = GD.playerMgr.Shock
        if (!bool) return;
        if (global.channel == CHANNEL.TT) {
            Toutiao.vibrateShort();
        } else if (global.channel == CHANNEL.WECHAT) {
            Wechat.vibrateShort();
        }
    }

}
