
import { _decorator, Node, ButtonComponent, Prefab, instantiate, Component, log } from 'cc';
const { ccclass, property } = _decorator;
@ccclass('SDKbaseView')
export class SDKbaseView extends Component {
    private _clickEnable: boolean = true; // 控制能否点击的变量
    /**
   * 绑定按钮点击事件,永久监听
   * @param node 挂载了button组件的节点
   * @param callback 回调
   * @param opname 操作内容
   */
    protected bindButton(node: Node, callback: (...parameters: any) => void, ...parameters: any[]): void {
        let btnComp = node.getComponent(ButtonComponent);
        if (!btnComp) {
            btnComp = node.addComponent(ButtonComponent);
            btnComp.transition = ButtonComponent.Transition.SCALE
        }
        btnComp.zoomScale = 0.9;
        let btnCb = (event: any) => {
            if (!this._clickEnable) return;
            this._clickEnable = false;
            this.scheduleOnce(() => {
                this._clickEnable = true;
            }, 0.3);
            callback.call(this, ...parameters);
            TyqSdk.outMain();
        };
        node.on("click", btnCb, this);
    }

    /**
    * 绑定按钮点击事件(只监听一次)
    * @param node 挂载了button组件的节点
    * @param callback 回调
    * @param opname 操作内容
    */
    protected bindButtonOce(node: Node, callback: (event?: any) => void, opname?: string): void {
        let btnComp = node.getComponent(ButtonComponent);
        if (!btnComp) {
            btnComp = node.addComponent(ButtonComponent);
            btnComp.transition = ButtonComponent.Transition.SCALE
        }
        btnComp.zoomScale = 0.9;
        let btnCb = (event: any) => {
            if (!this._clickEnable) return;
            this._clickEnable = false;
            this.scheduleOnce(() => {
                this._clickEnable = true;
            }, 0.3);
            callback.call(this, event);
            TyqSdk.outMain();
        };
        node.once("click", btnCb, this);
    }

    protected showUI(cPrefab: Prefab, cNode: Node, next?: any) {
        let node: Node = instantiate(cPrefab);
        node.parent = cNode;
        next && next(node.getComponent(node.name) ? node.getComponent(node.name) : node);
    }
}


